package de.unikoblenz.fgbks.base.utils.boundary.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.EXCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.BoundaryParseException;
import org.apache.commons.lang3.Validate;

public class IntegerBoundary extends AbstractGrowingBoundary<Integer> {

  public IntegerBoundary(String text) {
    super(text, Integer.TYPE);
  }

  @Override
  protected Integer getMinValue() {
    return Integer.MIN_VALUE;
  }

  @Override
  protected Integer getMaxValue() {
    return Integer.MAX_VALUE;
  }

  @Override
  protected void parse(String text) {
    try {
      text = Validate.notNull(text).trim();
      if (text.isEmpty()) {
        upperBound = getMaxValue();
        lowerBound = getMinValue();
        lowerBoundType = INCLUSIVE;
        upperBoundType = INCLUSIVE;
      } else if (text.startsWith("[") || text.startsWith("]")) {
        lowerBoundType = text.startsWith("[") ? INCLUSIVE : EXCLUSIVE;
        upperBoundType = text.endsWith("]") ? INCLUSIVE : EXCLUSIVE;
        text = text.replace('[', '\0').replace(']', '\0').trim();
        String[] n = text.split("\\.\\.");
        lowerBound = Integer.parseInt(n[0]);
        upperBound = Integer.parseInt(n[1]);
      } else if (text.startsWith(">")) {
        lowerBoundType = text.contains("=") ? INCLUSIVE : EXCLUSIVE;
        text = text.replace('=', '\0').replace('>', '\0').trim();
        lowerBound = Integer.parseInt(text);
        upperBound = getMaxValue();
        upperBoundType = INCLUSIVE;
      } else if (text.startsWith("<")) {
        upperBoundType = text.contains("=") ? INCLUSIVE : EXCLUSIVE;
        text = text.replace('=', '\0').replace('<', '\0').trim();
        upperBound = Integer.parseInt(text);
        lowerBound = getMinValue();
        lowerBoundType = INCLUSIVE;
      } else if (text.startsWith("=")) {
        text = text.replace('=', '\0').trim();
        upperBound = Integer.parseInt(text);
        lowerBound = Integer.parseInt(text);
        lowerBoundType = INCLUSIVE;
        upperBoundType = INCLUSIVE;
      } else {
        throw new BoundaryParseException();
      }
    } catch (Exception e) {
      throw new BoundaryParseException(
          String.format("Parsing boundary failed: %s. Exception: %s", text, e.getMessage()));
    }
    simplifyBound();
  }

  private void simplifyBound() {
    if (lowerBoundType == EXCLUSIVE) {
      lowerBound = lowerBound + 1;
      lowerBoundType = INCLUSIVE;
    }
    if (upperBoundType == EXCLUSIVE) {
      upperBound = upperBound - 1;
      upperBoundType = INCLUSIVE;
    }
  }
}
